"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.getAsyncRouter = exports.areLocationsEquivalent = void 0;
const jsx_runtime_1 = require("react/jsx-runtime");
const react_1 = require("react");
// eslint-disable-next-line no-restricted-imports
const react_router_dom_1 = require("react-router-dom");
const iti_react_core_1 = require("@interface-technologies/iti-react-core");
const components_1 = require("../components");
const ReadyContext_1 = require("./ReadyContext");
function areLocationsEquivalent(a, b) {
    return a.pathname === b.pathname && a.search === b.search;
}
exports.areLocationsEquivalent = areLocationsEquivalent;
/**
 * A component factory that returns an `AsyncRouter`. When using `AsyncRouter`
 * and a link changes the route, the new page is mounted but not displayed until
 * it has finished loading data and calls `onReady`. A progress bar at the top
 * of the page (usually NProgress) indicates to the user that the new page is
 * loading.
 *
 * It is fine if a page calls `onReady` multiple times.
 *
 * Example:
 * ```
 * export interface OnReadyArgs {
 *     title: string
 * }
 *
 * const AsyncRouter = getAsyncRouter<OnReadyArgs>()
 *
 * export function MyAsyncRouter(): React.ReactElement {
 *     const onReady = useCallback(
 *         ({ title }: OnReadyArgs): void => {
 *             updateTitle(title)
 *         },
 *         []
 *     )
 *
 *     const onInitialPageReady = useCallback((): void => {
 *         removeLoadingScreen()
 *     }, [])
 *
 *     return (
 *         <AsyncRouter
 *             renderRoutes={props => <AppRoutes {...props} />}
 *             renderLayout={(children) => (
 *                 <Layout>
 *                     {children}
 *                 </Layout>
 *             )}
 *             getLocationKey={getLocationKey}
 *             onNavigationStart={NProgress.start}
 *             onNavigationDone={NProgress.done}
 *             onReady={onReady}
 *             onInitialPageReady={onInitialPageReady}
 *         />
 *     )
 * }
 *
 * function getLocationKey(location: Location): string {
 *     return location.pathname.toLowerCase()
 * }
 * ```
 *
 * ### `getLocationKey`
 *
 * - If you want to be able to change the route parameters without the page
 *   unmounting and remounting, you should implement `getLocationKey`, so that
 *   the page has the same location key regardless of the route params.
 *
 *   Example: you want to be able to navigate from `/job/0` to `/job/1` without
 *   the page remounting. You should make `getLocationKey` return `'/job'` for
 *   any path that matches `'/job/:page?`. Use `matchPath` for this.
 *
 *   LocationKey is NOT the same as the `key` property of the `location` object.
 *
 * - When implementing `getLocationKey`, if the current path does not correspond
 *   to a route in your application, you must return the current path without
 *   any modifications.
 *
 *   Consider this example to see why you must return the path unmodified. Say
 *   you have a route `/job/:page?` and you have implemented `getLocationKey` to
 *   return `'/job'` for any path that starts with `/job`. The user enters
 *   `/job/bogus/path/here` into the URL bar. The "Page not found" page will be
 *   rendered. Then the user clicks the navbar link to the job list. The
 *   location key does not change when the location changes, so `AsyncRouter`
 *   does not unmount the previous page and mount the new page. But the two
 *   pages are actually different! So yeah weird stuff can happen if you
 *   implement `getLocationKey` like this. Always use `matchPath` in
 *   `getLocationKey`!
 *
 * ### Test Cases
 *
 * 1. Normal navigation.
 * 2. Double click a link - navigation should still occur like normal.
 * 3. Click a link and press browser back button while page is loading.
 * 4. Click a link and then click a different link while page is loading.
 * 5. Go to the "Spam onReady" page in `test-website` and follow the steps.
 * 6. Redirect to self: click the ITI React "logo" in `test-website` while on
 *    `/home`. The logo is a link to `/`. When the path is `/`, a redirect
 *    to `/home` is rendered, putting you back where you started.
 *
 * @typeParam TOnReadyArgs the arguments your pages will pass to `onReady`
 */
function getAsyncRouter() {
    return function AsyncRouter({ renderRoutes, renderLayout, getLocationKey, onInitialPageReady, onReady: propsOnReady, onNavigationStart: propsOnNavigationStart, onNavigationDone: propsOnNavigationDone, }) {
        const location = (0, react_router_dom_1.useLocation)();
        const [displayedLocation, setDisplayedLocation] = (0, react_1.useState)(location);
        const [loadingLocation, setLoadingLocation] = (0, react_1.useState)();
        const [initialLocationCalledOnReady, setInitialLocationCalledOnReady] = (0, react_1.useState)(false);
        // default to true since initial page is loading
        const [navigationInProgress, setNavigationInProgress] = (0, react_1.useState)(true);
        (0, react_1.useEffect)(() => {
            if (typeof displayedLocation !== 'undefined') {
                const locationChanged = !areLocationsEquivalent(displayedLocation, location);
                const locationKeyChanged = getLocationKey(displayedLocation) !== getLocationKey(location);
                if (locationChanged) {
                    if (locationKeyChanged) {
                        setLoadingLocation(location);
                    }
                    else {
                        // Should not reload page when location key is the same - this is the whole
                        // point of location keys
                        setDisplayedLocation(location);
                    }
                }
            }
        }, [displayedLocation, location, getLocationKey]);
        function onNavigationStart() {
            setNavigationInProgress(true);
            propsOnNavigationStart();
        }
        const onNavigationDone = (0, react_1.useCallback)(() => {
            setNavigationInProgress(false);
            setLoadingLocation(undefined);
            window.scrollTo(0, 0);
            propsOnNavigationDone();
        }, [propsOnNavigationDone]);
        const prevLocation = (0, iti_react_core_1.usePrevious)(location);
        (0, react_1.useEffect)(() => {
            if (!navigationInProgress &&
                prevLocation &&
                getLocationKey(location) !== getLocationKey(prevLocation)) {
                // normal navigation start
                onNavigationStart();
            }
        });
        (0, react_1.useEffect)(() => {
            if (loadingLocation &&
                getLocationKey(location) === getLocationKey(displayedLocation)) {
                // We got redirected to the page we're already on
                onNavigationDone();
            }
        });
        const onReady = (0, react_1.useCallback)((location, args) => {
            const isForLoadingLocation = loadingLocation && areLocationsEquivalent(location, loadingLocation);
            // ignore any unexpected calls to onReady.
            // if the user begins navigation to one page, but then interrupts the navigation by clicking
            // on a link, we can still get an onReady call from the first page. This call must be ignored,
            // or else weirdness will occur.
            //
            // this can also happen when a page calls onReady multiple times, for example, the page calls
            // onReady every time a query completes. Calling onReady multiple times is harmless, and as such,
            // no warning should be displayed.
            if (isForLoadingLocation || !initialLocationCalledOnReady) {
                const isInitialLoad = !initialLocationCalledOnReady;
                // normal navigation done
                setDisplayedLocation(location);
                setInitialLocationCalledOnReady(true);
                onNavigationDone();
                // Necessary to support dialogs that have links
                (0, components_1.cleanUpImproperlyClosedDialog)();
                propsOnReady(args);
                if (isInitialLoad) {
                    onInitialPageReady();
                }
            }
        }, [
            initialLocationCalledOnReady,
            loadingLocation,
            onInitialPageReady,
            onNavigationDone,
            propsOnReady,
        ]);
        const displayedReadyContextValue = (0, react_1.useMemo)(() => ({
            ready: initialLocationCalledOnReady,
            onReady: (args) => onReady(displayedLocation, args),
            location: displayedLocation,
        }), [onReady, initialLocationCalledOnReady, displayedLocation]);
        const loadingReadyContextValue = (0, react_1.useMemo)(() => ({
            ready: false,
            onReady: (args) => {
                if (!loadingLocation)
                    return;
                onReady(loadingLocation, args);
            },
            location: loadingLocation,
        }), [onReady, loadingLocation]);
        const pages = [
            (0, jsx_runtime_1.jsx)(ReadyContext_1.ReadyContext.Provider, { value: displayedReadyContextValue, children: renderRoutes(displayedLocation) }, getLocationKey(displayedLocation)),
        ];
        if (loadingLocation &&
            getLocationKey(loadingLocation) !== getLocationKey(displayedLocation)) {
            pages.push((0, jsx_runtime_1.jsx)(ReadyContext_1.ReadyContext.Provider, { value: loadingReadyContextValue, children: renderRoutes(loadingLocation) }, getLocationKey(loadingLocation)));
        }
        return renderLayout(pages);
    };
}
exports.getAsyncRouter = getAsyncRouter;
